"""Various helper functions for the order history plugin."""

from datetime import date, timedelta


def date_to_month(d: date) -> date:
    """Convert a date to the first day of the associated month."""
    return d.replace(day=1)


def date_to_quarter(d: date) -> date:
    """Convert a date to the first day of the associated quarter."""
    return d.replace(month=(((d.month - 1) // 3) * 3) + 1, day=1)


def date_to_year(d: date) -> date:
    """Convert a date to the first day of the associated year."""
    return d.replace(month=1, day=1)


def convert_date(d: date, period='M') -> str:
    """Return the associated date for a given date, for the provided time-period.

    Arguments:
        - d: The date to find the associated date for
        - period: The time period to use (e.g. 'W' for week, 'M' for month, 'Q' for quarter, 'Y' for year)
    """
    if not d:
        return None

    # Convert the date to the first day of the associated period (default = month)
    convert_func = {
        'M': date_to_month,
        'Q': date_to_quarter,
        'Y': date_to_year,
    }.get(period, date_to_month)

    return convert_func(d).isoformat().split('T')[0]


def construct_date_range(start_date: date, end_date: date, period='M') -> list:
    """Construct a list of date keys for the provided date range.

    Arguments:
        - start_date: The start date
        - end_date: The end date
        - period: The time period to use (e.g. 'W' for week, 'M' for month, 'Q' for quarter, 'Y' for year)

    Returns:
        A list of date keys for the provided date range
    """
    date_range = set()

    date_range.add(convert_date(start_date, period))

    date = start_date

    while date <= end_date:
        date_range.add(convert_date(date, period))
        date += timedelta(days=20)

    date_range.add(convert_date(end_date, period))

    return sorted(date_range, key=lambda x: x)



"""DRF API serializers for the Order History plugin."""

from django.utils.translation import gettext_lazy as _
from rest_framework import serializers

from company.models import Company, SupplierPart
from part.models import Part
from part.serializers import PartBriefSerializer


class OrderHistoryRequestSerializer(serializers.Serializer):
    """Serializer for requesting history data from the OrderHistory plugin."""

    class Meta:
        fields = [
            'start_date',
            'end_date',
            'period',
            'order_type',
            'part',
            'company',
            'supplier_part',
            'export'
        ]

    start_date = serializers.DateField(label=_('Start Date'), required=True)

    end_date = serializers.DateField(label=_('End Date'), required=True)

    period = serializers.ChoiceField(
        label=_('Period'),
        choices=[('M', _('Month')), ('Q', _('Quarter')), ('Y', _('Year'))],
        required=False,
        default='D',
        help_text=_('Group order data by this period'),
    )

    order_type = serializers.ChoiceField(
        label=_('Order Type'),
        choices=[('build', _('Build Order')), ('purchase', _('Purchase Order')), ('sales', _('Sales Order')), ('return', _('Return Order'))],
        help_text=_('Filter order data by this type'),
    )

    part = serializers.PrimaryKeyRelatedField(
        queryset=Part.objects.all(), many=False, required=False, label=_('Part')
    )

    supplier_part = serializers.PrimaryKeyRelatedField(
        queryset=SupplierPart.objects.all(), many=False, required=False, label=_('Supplier Part')
    )

    company = serializers.PrimaryKeyRelatedField(
        queryset=Company.objects.all(), many=False, required=False, label=_('Company')
    )

    export = serializers.ChoiceField(
        choices=[(choice, choice) for choice in ['csv', 'tsv', 'xls', 'xlsx']],
        required=False,
        label=_('Export Format')
    )


class OrderHistoryItemSerializer(serializers.Serializer):
    """Serializer for a single item in the OrderHistoryResponseSerializer."""

    class Meta:
        """Metaclass options for this serializer."""

        fields = ['date', 'quantity']

    date = serializers.DateField(read_only=True)
    quantity = serializers.FloatField(read_only=True)


class OrderHistoryResponseSerializer(serializers.Serializer):
    """Serializer for returning history data from the OrderHistory plugin."""

    class Meta:
        """Metaclass options for this serializer."""

        fields = ['part', 'history']

    part = PartBriefSerializer(read_only=True, many=False)
    history = OrderHistoryItemSerializer(many=True, read_only=True)
